
TOOL.Category		= "Poser"
TOOL.Name		= "#SetExpressionOverride"
TOOL.Command		= nil
TOOL.ConfigName		= ""

TOOL.ClientConVar[ "vcd" ] = ""
TOOL.ClientConVar[ "key" ] = "-1"

if (CLIENT) then

	language.Add( "SetExpressionOverride", "Play Scene")
	language.Add( "SetExpressionOverride_desc", "Force an NPC to play a premade scene")
	language.Add( "Tool_setexpressionoverride_name", "Play Scene" )
	language.Add( "Tool_setexpressionoverride_desc", "Force an NPC to play a premade scene" )
	language.Add( "Tool_setexpressionoverride_0", "Click on an NPC to play a scene. Right-click on an NPC to stop playing a scene." )
	language.Add( "SetExpressionOverride_Scene", "Scene:" )
	language.Add( "SetExpressionOverride_Scene_Custom", "Custom Scene:" )

end

local function CheckScene(ply, scene)
	if (string.lower(scene) == "none.vcd") then
		return
	end
	
	local scene = "../" .. scene
	
	if (file.Exists(scene)) then return end
	
	ply:PrintMessage(HUD_PRINTCONSOLE, "Scene \"" .. scene .. "\" not found!\n")
end

local function BeginScene(npc, scene)

// use a logic_choreographed_scene to time the scene length
// unfortunatly this can't be used to actually control the scene
// since it stops most of the scenes working without the correctly
// named actors around :(

	if (npc:GetTable().CurrentScene) then
		npc:GetTable().CurrentScene:Fire("Cancel", "", 0)
		npc:GetTable().CurrentScene:RemoveCallOnRemove("seo_remove_ref")
		npc:GetTable().CurrentScene:Remove()
		npc:GetTable().CurrentScene = nil
		npc:Fire("setexpressionoverride", "none.vcd", 0)
	end
	if (scene == "none.vcd") then
		return
	end	

	local LCS = ents.Create("logic_choreographed_scene")
	LCS:SetKeyValue("SceneFile", scene)
	LCS:Spawn()
	LCS:Activate()
	LCS:CallOnRemove("seo_remove_ref", 
				function(me,npc) 
					if (npc:IsValid()) then 
						npc:GetTable().CurrentScene = nil 
						npc:Fire("setexpressionoverride", "none.vcd", 0)
					end
				end, 
		npc )

	LCS:Fire("Start", "", 0)
	LCS:Fire("addoutput", "OnCompletion !self,Kill", 0)
	npc:GetTable().CurrentScene = LCS
	npc:DeleteOnRemove(LCS)
	npc:Fire("setexpressionoverride", scene, 0)
	
end

if (SERVER) then
	
	local function setexpressionoverride(ply, npc, key)

		if (!npc:IsValid()) then return false end
		if (!npc:GetTable().Scenes) then return false end

		local scene = npc:GetTable().Scenes[key] or "none.vcd";
		
		CheckScene(ply, scene)
		
		BeginScene(npc, scene)
	end
	numpad.Register( "setexpressionoverride", 	setexpressionoverride )
end

function TOOL:ApplyExpression(trace, scene, key)

	if trace.Entity && trace.Entity:IsPlayer() then return false end
	
	if (!trace.Entity:IsNPC()) then return false end

	if (CLIENT) then return true end

	local npc = trace.Entity
	if (key == -1) then
		CheckScene(self:GetOwner(), scene)
		BeginScene(npc, scene)
	else
		npc:GetTable().Scenes = npc:GetTable().Scenes or {}
		npc:GetTable().Scenes[key] = scene
	
		numpad.OnDown( self:GetOwner(), key, "setexpressionoverride", npc, key )
	end

	return true
end

function TOOL:LeftClick( trace )

	return self:ApplyExpression(trace, self:GetClientInfo("vcd"), self:GetClientNumber("key") )

end

function TOOL:RightClick( trace )

	return self:ApplyExpression(trace, "none.vcd", -1 )

end

if (CLIENT) then

	local function findScenes( dir, files_found )

		local files_in_dir = file.Find("../" .. dir .. "/*")
		local isTopLevel = files_found == nil
		files_found = files_found or {}

		for k,v in pairs(files_in_dir) do

			if (v != "." && v != "..") then

				local full_path = dir .. "/" .. v

				if file.IsDir( "../" .. full_path) then

					findScenes( full_path, files_found )

				elseif (string.upper( string.Right( full_path, 4 ) ) == ".VCD" ) then

					table.insert( files_found, full_path )

				end

			end

		end

		if (isTopLevel) then
			return files_found
		end
	end

	local scenes = {}
	
	if file.Exists("setexpressionoverride_scene_cache.txt") then
		local cache = file.Read("setexpressionoverride_scene_cache.txt")
		scenes = util.KeyValuesToTable(cache)
	else
		scenes = findScenes("scenes")
		file.Write("setexpressionoverride_scene_cache.txt", util.TableToKeyValues(scenes) )
	end
	
	local function BuildSEOPanel( Panel )
		// HEADER
		Panel:AddControl( "Header", { Text = "#SetExpressionOverride", Description	= "#SetExpressionOverride_desc" }  )

		// PRESETS
		local params = { Label = "#SetExpressionOverride_Scene", MenuButton = 0, Options = {}, Height = 300 }
		params.Options[ "< None >" ] = { ["ID"] = 0, ["setexpressionoverride_vcd"] = "none.vcd" }
		for k,v in pairs(scenes) do
			local name = v;
			name = string.sub(name, 8, -5)
			params.Options[ name ] = { ["ID"] = k, ["setexpressionoverride_vcd"] = v }
		end
		Panel:AddControl( "ListBox", params )

		Panel:AddControl( "TextBox", { Label = "#SetExpressionOverride_Scene_Custom", Command = "setexpressionoverride_vcd", WaitForEnter = 1 } )

		Panel:AddControl( "Numpad", { Label = "#Key", Command = "setexpressionoverride_key", ButtonSize = 22 } )
	end
	
	local ThePanelHack = nil
	
	function TOOL.BuildCPanel( Panel )
		ThePanelHack = Panel
		BuildSEOPanel(Panel)	
	end

	concommand.Add("setexpressionoverride_reset_cache", 
		function (ply, cmd, args)
			scenes = findScenes("scenes")
			file.Write("setexpressionoverride_scene_cache.txt", util.TableToKeyValues(scenes) )

			if (ThePanelHack) then			
				ThePanelHack:ClearControls()
				BuildSEOPanel(ThePanelHack)
			end
		end
	)

end
